<?php

namespace Realestate\Neighbourhood\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Realestate\Neighbourhood\Forms\Neighbourhood as NeighbourhoodForm;
use Realestate\Neighbourhood\Http\Requests\NeighbourhoodResourceRequest;
use Realestate\Neighbourhood\Http\Resources\NeighbourhoodResource;
use Realestate\Neighbourhood\Http\Resources\NeighbourhoodsCollection;
use Realestate\Neighbourhood\Models\Neighbourhood;
use Realestate\Neighbourhood\Actions\NeighbourhoodAction;
use Realestate\Neighbourhood\Actions\NeighbourhoodActions;


/**
 * Resource controller class for neighbourhood.
 */
class NeighbourhoodResourceController extends BaseController
{

    /**
     * Initialize neighbourhood resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = NeighbourhoodForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('realestate.neighbourhood.modules'), 'neighbourhood', guard_url('neighbourhood'));
            return $next($request);
        });
    }

    /**
     * Display a list of neighbourhood.
     *
     * @return Response
     */
    public function index(NeighbourhoodResourceRequest $request)
    {
        $request = $request->all();
        $page = NeighbourhoodActions::run('paginate', $request);

        $data = new NeighbourhoodsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('neighbourhood::neighbourhood.names'))
            ->view('neighbourhood::neighbourhood.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display neighbourhood.
     *
     * @param Request $request
     * @param Model   $neighbourhood
     *
     * @return Response
     */
    public function show(NeighbourhoodResourceRequest $request, Neighbourhood $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new NeighbourhoodResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('neighbourhood::neighbourhood.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('neighbourhood::neighbourhood.show')
            ->output();
    }

    /**
     * Show the form for creating a new neighbourhood.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(NeighbourhoodResourceRequest $request, Neighbourhood $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new NeighbourhoodResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('neighbourhood::neighbourhood.name'))
            ->view('neighbourhood::neighbourhood.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new neighbourhood.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(NeighbourhoodResourceRequest $request, Neighbourhood $model)
    {
        try {
            $request = $request->all();
            $model = NeighbourhoodAction::run('store', $model, $request);
            $data = new NeighbourhoodResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('neighbourhood::neighbourhood.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('neighbourhood/neighbourhood/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/neighbourhood/neighbourhood'))
                ->redirect();
        }

    }

    /**
     * Show neighbourhood for editing.
     *
     * @param Request $request
     * @param Model   $neighbourhood
     *
     * @return Response
     */
    public function edit(NeighbourhoodResourceRequest $request, Neighbourhood $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new NeighbourhoodResource($model);
        // return view('neighbourhood::neighbourhood.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('neighbourhood::neighbourhood.name'))
            ->view('neighbourhood::neighbourhood.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the neighbourhood.
     *
     * @param Request $request
     * @param Model   $neighbourhood
     *
     * @return Response
     */
    public function update(NeighbourhoodResourceRequest $request, Neighbourhood $model)
    {
        try {
            $request = $request->all();
            $model = NeighbourhoodAction::run('update', $model, $request);
            $data = new NeighbourhoodResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('neighbourhood::neighbourhood.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('neighbourhood/neighbourhood/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('neighbourhood/neighbourhood/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the neighbourhood.
     *
     * @param Model   $neighbourhood
     *
     * @return Response
     */
    public function destroy(NeighbourhoodResourceRequest $request, Neighbourhood $model)
    {
        try {

            $request = $request->all();
            $model = NeighbourhoodAction::run('destroy', $model, $request);
            $data = new NeighbourhoodResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('neighbourhood::neighbourhood.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('neighbourhood/neighbourhood/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('neighbourhood/neighbourhood/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
