<?php

namespace Realestate\Neighbourhood\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Realestate\Neighbourhood\Forms\Sections as SectionsForm;
use Realestate\Neighbourhood\Http\Requests\SectionsResourceRequest;
use Realestate\Neighbourhood\Http\Resources\SectionsResource;
use Realestate\Neighbourhood\Http\Resources\SectionsCollection;
use Realestate\Neighbourhood\Models\Sections;
use Realestate\Neighbourhood\Actions\SectionsAction;
use Realestate\Neighbourhood\Actions\SectionsActions;


/**
 * Resource controller class for sections.
 */
class SectionsResourceController extends BaseController
{

    /**
     * Initialize sections resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = SectionsForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('realestate.neighbourhood.modules'), 'neighbourhood', guard_url('neighbourhood'));
            return $next($request);
        });
    }

    /**
     * Display a list of sections.
     *
     * @return Response
     */
    public function index(SectionsResourceRequest $request)
    {
        $request = $request->all();
        $page = SectionsActions::run('paginate', $request);

        $data = new SectionsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('neighbourhood::sections.names'))
            ->view('neighbourhood::sections.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display sections.
     *
     * @param Request $request
     * @param Model   $sections
     *
     * @return Response
     */
    public function show(SectionsResourceRequest $request, Sections $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new SectionsResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('neighbourhood::sections.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('neighbourhood::sections.show')
            ->output();
    }

    /**
     * Show the form for creating a new sections.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SectionsResourceRequest $request, Sections $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new SectionsResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('neighbourhood::sections.name'))
            ->view('neighbourhood::sections.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new sections.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SectionsResourceRequest $request, Sections $model)
    {
        try {
            $request = $request->all();
            $model = SectionsAction::run('store', $model, $request);
            $data = new SectionsResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('neighbourhood::sections.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('neighbourhood/sections/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/neighbourhood/sections'))
                ->redirect();
        }

    }

    /**
     * Show sections for editing.
     *
     * @param Request $request
     * @param Model   $sections
     *
     * @return Response
     */
    public function edit(SectionsResourceRequest $request, Sections $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new SectionsResource($model);
        // return view('neighbourhood::sections.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('neighbourhood::sections.name'))
            ->view('neighbourhood::sections.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the sections.
     *
     * @param Request $request
     * @param Model   $sections
     *
     * @return Response
     */
    public function update(SectionsResourceRequest $request, Sections $model)
    {
        try {
            $request = $request->all();
            $model = SectionsAction::run('update', $model, $request);
            $data = new SectionsResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('neighbourhood::sections.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('neighbourhood/sections/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('neighbourhood/sections/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the sections.
     *
     * @param Model   $sections
     *
     * @return Response
     */
    public function destroy(SectionsResourceRequest $request, Sections $model)
    {
        try {

            $request = $request->all();
            $model = SectionsAction::run('destroy', $model, $request);
            $data = new SectionsResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('neighbourhood::sections.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('neighbourhood/sections/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('neighbourhood/sections/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
